<?php
/**
 * This file contains the FEText, FEName, FEEmail,
 * FEDomainName classes
 *
 * $Id: FEText.inc,v 1.17 2003/07/18 01:23:43 hemna Exp $
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markosyan <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 *
 */

/**
 * This is the Text FormElement which builds a 
 * text input field. It has no validation method.
 * 
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markossian <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 */
class FEText extends FormElement {

    /**
     * The constructor
     *
     * @param label string - text label for the element
     * @param bool required - is this a required element
     * @param int required - element width in characters, pixels (px), percentage (%) or elements (em)
     * @param int required - maximum number of chars allowed to type in
     */
    function FEText($label, $required = TRUE, $width = NULL, $maxlength = NULL) {
        $this->FormElement($label, $required);

        if ($width != NULL) {
            if (is_numeric($width))
                $this->set_attribute("size", $width);
            else
                $this->set_style_attribute("width", $width);
        }

        if ($maxlength != NULL)
            $this->set_attribute("maxlength", $maxlength);
    }


    /**
     * This function builds and returns the
     * form element object
     *
     * @return object
     */
    function get_element() {

        $attributes = $this->_build_element_attributes();
        $attributes["type"] = "text";

        if (($value = $this->get_value()) != NULL)
            $attributes["value"] = $value;

        return new INPUTtag($attributes);
    }
}

/**
 * This is the Name FormElement which builds a 
 * text input field, and validates against the
 * is_name() method.
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markossian <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 */
class FEName extends FEText {

    /**
     * This method validates the data
     * for this Form Element.
     *
     * It validates as is_name().
     * @param FormValidation object.
     */
    function validate(&$_FormValidation) {
        if (!$_FormValidation->is_name($this->get_value())) {
            $this->set_error_message( $_FormValidation->get_error_message() );
            return FALSE;
        }
        return TRUE;
    }
}

/**
 * This is the Email FormElement which builds a 
 * text input field. 
 * It validatest he data as is_email()
 * 
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markossian <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 */
class FEEmail extends FEText {

    /**
     * This method validates the data
     * for this Form Element.
     *
     * It validates as is_email().
     * @param FormValidation object.
     */
    function validate($_FormValidation) {
        if (!$_FormValidation->is_email($this->get_value())) {
            $this->set_error_message( $_FormValidation->get_error_message() );
            return FALSE;
        }
        return TRUE;
    }
}

/**
 * This is the EmailMany FormElement which builds a 
 * text input field.  This allows for multiple email
 * addresses in 1 text input field seperated by commas.
 *
 * It validatest he data as is_manyemails()
 * 
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markossian <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 */
class FEEmailMany extends FEText {

    /**
     * This method validates the data
     * for this Form Element.
     *
     * It validates as is_manyemails().
     * @param FormValidation object.
     */
    function validate(&$_FormValidation) {
        if (!$_FormValidation->is_manyemails($this->get_value())) {
            $this->set_error_message( $_FormValidation->get_error_message() );
            return FALSE;
        }
        return TRUE;
    }
}

/**
 * This is the DomainName FormElement which builds a 
 * text input field.
 * It validates as is_domainname().
 * 
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markossian <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 */
class FEDomainName extends FEText {

    /**
     * This method validates the data
     * for this Form Element.
     *
     * It validates as is_domainname().
     * @param FormValidation object.
     */
    function validate(&$_FormValidation) {
        if (!$_FormValidation->is_domainname($this->get_value())) {
            $this->set_error_message( $_FormValidation->get_error_message() );
            return FALSE;
        }
        return TRUE;
    }
}

/**
 * This is the DomainName FormElement which builds a 
 * text input field. It also includes a port #
 * It validates as is_domainname().
 * 
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markossian <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 */
class FEHostNameWithPort extends FEText {

    /**
     * flag to tell us to seperate the port 
     * value into it's own field
     */
    var $_seperate_port_flag = FALSE;

    /**
     * The constructor
     *
     * @param label string - text label for the element
     * @param bool required - is this a required element
     * @param int required - element width in characters, pixels (px), percentage (%) or elements (em)
     * @param int required - maximum number of chars allowed to type in
     * @param bool optional - seperate the port value into it's own textarea field
     */
    function FEHostNameWithPort($label, $required = TRUE, $width = NULL, $maxlength = NULL,
                                  $seperate_port=FALSE) {
        $this->FEText($label, $required, $width, $maxlength);
        $this->_seperate_port_flag = $seperate_port;
    }

    /**
     * This function builds and returns the
     * form element object
     *
     * @return object
     */
    function get_element() {
        $host_attributes = $this->_build_element_attributes();
        $host_attributes["type"] = "text";

        $element_name = $this->get_element_name();                
        $host_value = $this->get_value();

        if ($this->_seperate_port_flag) {
            $host_attributes["name"] = $element_name."[0]";
            if ($host_value != NULL && !is_array($host_value)) {
                $host_value = explode(":", $host_value );
                $host_attributes["value"] = $host_value[0];
            } else if (is_array($host_value)) {
                $host_attributes["value"] = $host_value[0];
            }

            $port_attributes = $host_attributes;
            $port_attributes["name"] = $element_name."[1]";
            $port_attributes["size"] = 10;
            $port_attributes["maxlenght"] = 5;
            if (@array_key_exists("0", $host_value)) {
                $port_attributes["value"] = $host_value[1];
            }

            $port_label = html_span("formlabel", "Port");
            if ($this->has_error("Port")) {
                $port_label->set_tag_attribute("style","color:red;");
            }            

            return container( new INPUTtag($host_attributes),
                              $port_label,
                              new INPUTtag($port_attributes));
        } else {
            if ($host_value != NULL) {
                $host_attributes["value"] = $host_value;
            }
            return new INPUTtag($host_attributes);
        }
    }

    /**
     * This method validates the data
     * for this Form Element.
     *
     * It validates as is_domainname().
     * @param FormValidation object.
     */
    function validate(&$_FormValidation) {
        $value = $this->get_value();
        if (!is_array($value)) {
            $value = explode(":", $value);
        }
        
        $valid = $this->_validate_hostname($_FormValidation, $value[0]);

        //now validate the port
        if ($value[1] != NULL) {
            if (!$_FormValidation->is_number($value[1])) {
                if ($this->_seperate_port_flag) {
                    $this->set_error_message($_FormValidation->get_error_message(), "Port");
                } else {
                    $this->set_error_message( $_FormValidation->get_error_message() );
                }
                $value = FALSE;
            }

            //make sure the hostname isn't null
            if ($value[0] == NULL) {
                //we have an error here because
                //they entered a port w/o a hostname
                $_FormValidation->_error("", "This field cannot be empty, if the Port is filled out");
                $this->set_error_message($_FormValidation->get_error_message());
            }
        }
        return $valid;
    }


    /**
     * this validates the hostname field only
     * 
     * @return bool TRUE = valid
     */
    function _validate_hostname(&$_FormValidation, $value) {
        $valid = TRUE;
        //validate the hostname        
        if ($value != NULL && !$_FormValidation->is_hostname($value)) {
            $this->set_error_message( $_FormValidation->get_error_message() );
            $valid = FALSE;
        }

        return $valid;
    }
}

/**
 * This is the Ip Address FormElement which builds a 
 * text input field.
 * It validates as is_ip().
 * 
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markossian <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 */
class FEIPAddress extends FEText {

    /**
     * This method validates the data
     * for this Form Element.
     *
     * It validates as is_ip().
     * @param FormValidation object.
     */
    function validate(&$_FormValidation) {
        if (!$_FormValidation->is_ip($this->get_value())) {
            $this->set_error_message( $_FormValidation->get_error_message() );
            return FALSE;
        }
        return TRUE;
    }
}

/**
 * This is the Ip Address FormElement which builds a 
 * text input field.
 * It validates as is_ip().
 * 
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markossian <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 */
class FEIPAddressWithPort extends FEHostNameWithPort {

    /**
     * this validates the hostname field only
     * 
     * @return bool TRUE = valid
     */
    function _validate_hostname(&$_FormValidation, $value) {
        $valid = TRUE;
        //validate the hostname        
        if ($value != NULL && !$_FormValidation->is_ip($value)) {
            $this->set_error_message( $_FormValidation->get_error_message() );
            $valid = FALSE;
        }
        return $valid;
    }
}



/**
 * This is the URL FormElement which builds a 
 * text input field.
 * It validates as is_url().
 * 
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markossian <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 */
class FEUrl extends FEText {

    /**
     * This method validates the data
     * for this Form Element.
     *
     * It validates as is_url().
     * @param FormValidation object.
     */
    function validate(&$_FormValidation) {
        if (!$_FormValidation->is_url($this->get_value())) {
            $this->set_error_message( $_FormValidation->get_error_message() );
            return FALSE;
        }
        return TRUE;
    }
}

/**
 * This is the URLStrict FormElement which builds a 
 * text input field.
 * This is the same as FEUrl, but it requires the http://
 * It validates as is_strict_url().
 * 
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @author Suren Markossian <suren@bcsweb.com>
 * @package phpHtmlLib
 * @subpackage FormProcessing
 *
 * @copyright LGPL - See LICENCE
 */
class FEUrlStrict extends FEText {

    /**
     * This method validates the data
     * for this Form Element.
     *
     * It validates as is_strict_url().
     * @param FormValidation object.
     */
    function validate(&$_FormValidation) {
        if (!$_FormValidation->is_strict_url($this->get_value())) {
            $this->set_error_message( $_FormValidation->get_error_message() );
            return FALSE;
        }
        return TRUE;
    }
}
?>
