<?php
/**
 * $Id: ArrayDataListSource.inc,v 1.2 2003/07/31 22:11:44 hemna Exp $
 *
 * This file contains the DataListSource class
 * that gets the data from an array
 *
 * @author Suren Markossian <suren_markossian@yahoo.com>
 * @package phpHtmlLib
 */




/**
 *
 * This DataListSource child class gets the data from an
 * external array
 *
 * The array is a 2D array and each sub-array is a set of elements for
 * each row in the list
 *
 * @author Suren Markossian <suren_markossian@yahoo.com>
 * @package phpHtmlLib
 */
class ArrayDataListSource extends DataListSource {


    /**
     * If true the prequery filter will be
     * run on the data array
     */
    var $_prequery_filter_flag = FALSE;

    /**
     * The constructor.
     *
     * @param array data - an array of array elements for each row
     */
    function ArrayDataListSource(&$data) {
        $this->_data = $data;
    }

    /**
     * This function does the query
     * and search/sort
     */
    function do_query() {

        // do data pre-processing if neccessary
        if ($this->_prequery_filter_flag) $this->_prequery_filter();

        $count = count($this->_data);

        if ($count > 0) {
            $this->set_total_rows($count);
            $this->sort();
            return true;
        } else {
            return false;
        }
    }

    /**
     * This is a method that should be defined by the
     * child class to do any pre-query type of things.
     * Such as building a sql query string for a DB,
     * or checking to make sure the file on disk exists
     * if the source is a file on disk.
     *
     */
    function do_prequery() {
        return NULL;
    }

    /**
     * This function returns the next row of
     * valid data.
     *
     */
    function get_next_data_row() {
        $index = $this->get_data_index();
        $limit = $this->get_limit();
        $offset = $this->get_offset();

        if ($limit == -1) {
            // don't limit the data
            if ($index > $this->get_total_rows()) {
                return NULL;
            } else {
                return $this->_data[$index];
            }
        } else {
            if (isset($this->_data[$index])) {
                return $this->_data[$index];
            } else {
                return NULL;
            }
        }
    }

    /**
     * This is used to perform pre-query filtering
     * Gives us a chance to run the next row through a filter
     * before any processing has been done
     *
     * @param array - the row to run through the filter
     *
     * @return boolean - TRUE = allow the row. FALSE = drop it.
     */
    function prequery_row_filter(&$row_data) {
        return TRUE;
    }

    /**
     * This is called to allow rebuilding the data array
     * to remove elements that have to be filtered
     *
     * This is done on pre-processing phase and used for
     * simple and advanced searches
     *
     */
    function _prequery_filter() {

        while (list($key,$row_data) = each($this->_data)) {
            if (!$this->prequery_row_filter($row_data)) {
                unset($this->_data[$key]);
            }
        }

        // re-index the array
        $this->_data = array_values($this->_data);
    }

    /**
     * Sets the pre-query filter flag
     * If true, a filter will be run on the data array
     * before any processing is done
     *
     * @param bool flag
     */
    function set_prequery_filter_flag($flag) {
        $this->_prequery_filter_flag = $flag;

    }

    /**
     * This function returns the
     * data_index value and increments it
     *
     * @return int
     */
    function get_data_index() {
        $this->_data_index = key($this->_data);
        next($this->_data);
        return $this->_data_index;
    }

    function sort() {
        $sortby = $this->get_orderby();
        // we need to sort the data by the
        // column in the table
        if ($this->_is_column_sortable($sortby)) {
            // looks like we can sort this column
            usort($this->_data, array($this, "cmp"));
        }
    }

    function cmp($data1, $data2) {
        $sortby = $this->get_orderby();
        $ret = strnatcmp( $data1[$sortby], $data2[$sortby]);
        if ($this->get_reverseorder() == "true") {
            $ret *= -1;
        }
        return $ret;
    }

}
?>
