<?php
/**
 * This file contains the DataListSource class
 * that gets the data from a CSV formatted file.  
 *
 *
 * $Id: CSVFILEDataListSource.inc,v 1.3 2003/04/29 06:38:05 hemna Exp $
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @package phpHtmlLib
 */



/**
 * 
 * This DataListSource child class gets the data from a 
 * CSV (comma seperated values) file on disk.
 *
 * The CSV MUST have a 'header' line before any data.
 * The 'header' line is a CSV line that provides a name
 * for each column of data in the file.
 *
 * All lines that start with '#' or '//' are ignored as 
 * comments.
 *
 *
 * @author Walter A. Boring IV <waboring@buildabetterweb.com>
 * @package phpHtmlLib 
 */
class CSVFILEDataListSource extends DataListSource {


	/**
	 * The csv file name/path on disk
	 *
	 */
	var $_filename = NULL;

	/**
	 * The file descriptor pointer
	 *
	 */
	var $_fp = NULL;


	/**
	 * this holds the headers read
	 * from the top of the csv file
	 */
	var $_csv_headers = array();

	/** 
	 * The length of the longest 
	 * line in the csv file
	 * (so we read the line properly)
	 */
	var $_maxlinelength=4096;
	 


	/**
	 * The constructor.
	 *
	 * @param string - the path to the CSV file on
	 *                 on disk to use as the data source.
	 *
	 */
	function CSVFILEDataListSource( $filename, $maxline=4096 ) {
		if (!file_exists($filename)) {
			user_error("CSVFILEDataListSource:: (".$filename.") ".
					   "can't be found.");
		} else {
			$this->_filename = $filename;
			$this->_fp = fopen( $filename, 'r');
			if (!$this->_fp) {
				user_error("CSVFILEDataListSource:: (".$filename.") ".
						   "cannot be opened");
			}
		}
		$this->_maxlinelength = $maxline;
	}


	/**
	 * The prequery.  We use this to read the file
	 * into memory so we can do operations on the data
	 * (search, sort, etc.)
	 */
	function do_prequery() {
		$this->_get_header();
	}

	/**
	 * This function does the query
	 * and search/sort
	 */
	function do_query() {
        $count = 0;
		while ($line = fgets($this->_fp, $this->_maxlinelength)) {
			if ($this->add_data_row( $this->_construct_row(trim($line)))) {
				$count++;
			}
		}

		//close the file
		fclose($this->_fp);
        if ($count > 0) {
            $this->set_total_rows( $count );
            $this->sort();
            return true;
        } else {
            return false;
        }
	}

	/**
	 * This function returns the next row of 
	 * valid data.  
	 *
	 */
	function get_next_data_row() {
		$index = $this->get_data_index();
		$limit = $this->get_limit();
		$offset = $this->get_offset();

		if ($limit == -1) {
			//don't limit the data
			if ($index > $this->get_total_rows()) {
				return NULL;
			} else {
				return $this->_data[$index];
			}
		} else {
			$left_to_show = $limit - ($index - $offset);
			if ($left_to_show > 0) {
				return $this->_data[$index];
			} else {
				return NULL;
			}
		}
	}


	/**
	 * This file trys to get the CSV header.
	 *
	 */
	function _get_header() {
		$done = FALSE;
		while (!$done) {
			$line = trim(fgets($this->_fp,$this->_maxlinelength));
			if (strncmp('#', $line, 1) != 0 &&
				strncmp('//', $line, 2) != 0 && !empty($line)) {
				$this->_csv_headers = explode(',', $line);
				$done = TRUE;
			}             
		}
	}

	
	/**
	 * this is used to build a row
	 * from a csv line
	 *
	 * @param string - the original csv line from the file
	 * @return array
	 */
	function _construct_row( $line ) {
		$row = NULL;
		if (strncmp('#', $line, 1) != 0 && 
			strncmp('//', $line, 2) != 0 && !empty($line)) {
			$tmp = explode(",", $line);
			//xmp_var_dump( $row );
			$row = array();
			foreach( $this->_csv_headers as $index => $name ) {
				$row[$name] = $tmp[$index];
			}
		}

		//xmp_var_dump( $row );
		return $row;
	}
	 


	/**
	 * This function adds a row of data
	 * if necesarry to the data array
	 *
	 */
	function add_data_row($row) {
		if ($row != NULL) {
			if ($this->get_searchby_value()) {
				//user wants to search the data
				if ($this->_find_data($row) ) {
					$this->_data[] = $row;
					return TRUE;
				} else {
					return FALSE;
				}
			} else {
				$this->_data[] = $row;
				return TRUE;
			}
		} else {
			return FALSE;
		}
	}

	function sort() {
		$sortby = $this->get_orderby();
		//ok we need to sort the data by the
		//column in the table
		if ($this->_is_column_sortable($sortby)) {
			//looks like we can sort this column
			usort($this->_data, array($this, "cmp"));
		}
	}

	function cmp($data1, $data2) {
		$sortby = $this->get_orderby();
		$ret = strnatcmp( $data1[$sortby], $data2[$sortby]);
		if ($this->get_reverseorder() == "true") {
			$ret *= -1;
		}
		return $ret;
	}

	function _find_data( $row_data ) {
		//look for the 'needle' in the 'haystack'

		$needle = $this->get_searchby_value();
		$haystack = $row_data[$this->get_searchby()];

		switch ($this->get_simplesearch_modifier()) {
		case "BEGINS":
			if (strncasecmp($needle, $haystack, strlen($needle)) == 0) {
				return TRUE;
			} else {
				return FALSE;
			}
			break;

		case "CONTAINS":
			if (stristr($haystack, $needle)) {
				return TRUE;
			} else {
				return FALSE;
			}
			break;
		case "EXACT":
			if (strcasecmp($needle, $haystack) == 0) {
				return TRUE;
			} else {
				return FALSE;
			}
			break;
		case "ENDS":
			$needle_len = strlen( $needle );
			$haystack_len = strlen( $haystack );
			if ($needle_len > $haystack_len) {
				return FALSE;
			} else if ($needle_len == $haystack_len) {
				$tmp = $haystack;
			} else {
				$tmp = substr($haystack, $haystack_len - $needle_len);
			}
			if (strncasecmp($needle, $tmp, $needle_len) == 0) {
				return TRUE;
			} else {
				return FALSE;
			}
			break;
		}
	}
	
}
?>
